@@ -40,12 +40,13 @@ import org.junit.Before;
 import org.junit.Test;
 
 import java.io.IOException;
-import java.util.ArrayList;
-import java.util.List;
 import java.util.NavigableSet;
 import java.util.Random;
 import org.slf4j.event.Level;
 
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys
+    .OZONE_SCM_DB_CACHE_SIZE_DEFAULT;
+
 /**
  * Tests for ContainerStateManager.
  */
@@ -55,7 +56,7 @@ public class TestContainerStateManagerIntegration {
   private MiniOzoneCluster cluster;
   private XceiverClientManager xceiverClientManager;
   private StorageContainerManager scm;
-  private ContainerMapping scmContainerMapping;
+  private ContainerManager containerManager;
   private ContainerStateManager containerStateManager;
   private PipelineSelector selector;
   private String containerOwner = "OZONE";
@@ -66,11 +67,12 @@ public class TestContainerStateManagerIntegration {
     conf = new OzoneConfiguration();
     cluster = MiniOzoneCluster.newBuilder(conf).setNumDatanodes(1).build();
     cluster.waitForClusterToBeReady();
+    cluster.waitTobeOutOfChillMode();
     xceiverClientManager = new XceiverClientManager(conf);
     scm = cluster.getStorageContainerManager();
-    scmContainerMapping = (ContainerMapping) scm.getScmContainerManager();
-    containerStateManager = scmContainerMapping.getStateManager();
-    selector = scmContainerMapping.getPipelineSelector();
+    containerManager = scm.getContainerManager();
+    containerStateManager = containerManager.getStateManager();
+    selector = containerManager.getPipelineSelector();
   }
 
   @After
@@ -119,24 +121,25 @@ public class TestContainerStateManagerIntegration {
   public void testContainerStateManagerRestart() throws IOException {
     // Allocate 5 containers in ALLOCATED state and 5 in CREATING state
 
-    List<ContainerInfo> containers = new ArrayList<>();
     for (int i = 0; i < 10; i++) {
       ContainerWithPipeline container = scm.getClientProtocolServer()
           .allocateContainer(
               xceiverClientManager.getType(),
               xceiverClientManager.getFactor(), containerOwner);
-      containers.add(container.getContainerInfo());
       if (i >= 5) {
-        scm.getScmContainerManager().updateContainerState(container
+        scm.getContainerManager().updateContainerState(container
                 .getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
       }
     }
 
+    cluster.getStorageContainerManager().stop();
     // New instance of ContainerStateManager should load all the containers in
     // container store.
     ContainerStateManager stateManager =
-        new ContainerStateManager(conf, scmContainerMapping, selector);
+        new SCMContainerManager(conf, scm.getScmNodeManager(),
+            OZONE_SCM_DB_CACHE_SIZE_DEFAULT,
+            scm.getEventQueue()).getStateManager();
     int matchCount = stateManager
         .getMatchingContainerIDs(containerOwner,
             xceiverClientManager.getType(), xceiverClientManager.getFactor(),
@@ -153,10 +156,10 @@ public class TestContainerStateManagerIntegration {
     ContainerWithPipeline container1 = scm.getClientProtocolServer().
         allocateContainer(xceiverClientManager.getType(),
             xceiverClientManager.getFactor(), containerOwner);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATED);
 
@@ -178,10 +181,10 @@ public class TestContainerStateManagerIntegration {
     Assert.assertEquals(container2.getContainerInfo().getContainerID(),
         info.getContainerID());
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container2.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container2.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATED);
 
@@ -215,7 +218,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.ALLOCATED).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -223,7 +226,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.CREATING).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATED);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -231,7 +234,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.OPEN).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.FINALIZE);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -239,7 +242,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.CLOSING).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CLOSE);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -247,7 +250,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.CLOSED).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.DELETE);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -255,7 +258,7 @@ public class TestContainerStateManagerIntegration {
         HddsProtos.LifeCycleState.DELETING).size();
     Assert.assertEquals(1, containers);
 
-    scmContainerMapping
+    containerManager
         .updateContainerState(container1.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CLEANUP);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -269,10 +272,10 @@ public class TestContainerStateManagerIntegration {
         .allocateContainer(
             xceiverClientManager.getType(),
             xceiverClientManager.getFactor(), containerOwner);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container2.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container2.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.TIMEOUT);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -286,16 +289,16 @@ public class TestContainerStateManagerIntegration {
         .allocateContainer(
             xceiverClientManager.getType(),
             xceiverClientManager.getFactor(), containerOwner);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container3.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATE);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container3.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CREATED);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container3.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.FINALIZE);
-    scmContainerMapping
+    containerManager
         .updateContainerState(container3.getContainerInfo().getContainerID(),
             HddsProtos.LifeCycleEvent.CLOSE);
     containers = containerStateManager.getMatchingContainerIDs(containerOwner,
@@ -309,10 +312,10 @@ public class TestContainerStateManagerIntegration {
     ContainerWithPipeline container1 = scm.getClientProtocolServer()
         .allocateContainer(xceiverClientManager.getType(),
         xceiverClientManager.getFactor(), containerOwner);
-    scmContainerMapping.updateContainerState(container1
+    containerManager.updateContainerState(container1
             .getContainerInfo().getContainerID(),
         HddsProtos.LifeCycleEvent.CREATE);
-    scmContainerMapping.updateContainerState(container1
+    containerManager.updateContainerState(container1
             .getContainerInfo().getContainerID(),
         HddsProtos.LifeCycleEvent.CREATED);
 
@@ -329,8 +332,8 @@ public class TestContainerStateManagerIntegration {
       Assert.assertEquals(container1.getContainerInfo().getContainerID(),
           info.getContainerID());
 
-      ContainerMapping containerMapping =
-          (ContainerMapping) scmContainerMapping;
+      SCMContainerManager containerMapping =
+          (SCMContainerManager) containerManager;
       // manually trigger a flush, this will persist the allocated bytes value
       // to disk
       containerMapping.flushContainerInfo();
